@@ -7,21 +7,17 @@
 import bdb
 import builtins
 import os
-import signal
-import subprocess
 import sys
-import time
-import warnings
-from subprocess import PIPE, CalledProcessError, check_output
+import platform
+
 from tempfile import NamedTemporaryFile
 from textwrap import dedent
 from unittest.mock import patch
 
-import nose.tools as nt
-
 from IPython.core import debugger
 from IPython.testing import IPYTHON_TESTING_TIMEOUT_SCALE
 from IPython.testing.decorators import skip_win32
+import pytest
 
 #-----------------------------------------------------------------------------
 # Helper classes, from CPython's Pdb test suite
@@ -59,23 +55,6 @@ class PdbTestInput(object):
 # Tests
 #-----------------------------------------------------------------------------
 
-def test_longer_repr():
-    try:
-        from reprlib import repr as trepr  # Py 3
-    except ImportError:
-        from repr import repr as trepr  # Py 2
-    
-    a = '1234567890'* 7
-    ar = "'1234567890123456789012345678901234567890123456789012345678901234567890'"
-    a_trunc = "'123456789012...8901234567890'"
-    nt.assert_equal(trepr(a), a_trunc)
-    # The creation of our tracer modifies the repr module's repr function
-    # in-place, since that global is used directly by the stdlib's pdb module.
-    with warnings.catch_warnings():
-        warnings.simplefilter('ignore', DeprecationWarning)
-        debugger.Tracer()
-    nt.assert_equal(trepr(a), ar)
-
 def test_ipdb_magics():
     '''Test calling some IPython magics from ipdb.
 
@@ -246,18 +225,22 @@ def test_interruptible_core_debugger():
     """
     def raising_input(msg="", called=[0]):
         called[0] += 1
-        if called[0] == 1:
-            raise KeyboardInterrupt()
-        else:
-            raise AssertionError("input() should only be called once!")
+        assert called[0] == 1, "input() should only be called once!"
+        raise KeyboardInterrupt()
+
+    tracer_orig = sys.gettrace()
+    try:
+        with patch.object(builtins, "input", raising_input):
+            debugger.InterruptiblePdb().set_trace()
+            # The way this test will fail is by set_trace() never exiting,
+            # resulting in a timeout by the test runner. The alternative
+            # implementation would involve a subprocess, but that adds issues
+            # with interrupting subprocesses that are rather complex, so it's
+            # simpler just to do it this way.
+    finally:
+        # restore the original trace function
+        sys.settrace(tracer_orig)
 
-    with patch.object(builtins, "input", raising_input):
-        debugger.InterruptiblePdb().set_trace()
-        # The way this test will fail is by set_trace() never exiting,
-        # resulting in a timeout by the test runner. The alternative
-        # implementation would involve a subprocess, but that adds issues with
-        # interrupting subprocesses that are rather complex, so it's simpler
-        # just to do it this way.
 
 @skip_win32
 def test_xmode_skip():
@@ -326,6 +309,197 @@ def test_xmode_skip():
     child.close()
 
 
+skip_decorators_blocks = (
+    """
+    def helpers_helper():
+        pass # should not stop here except breakpoint
+    """,
+    """
+    def helper_1():
+        helpers_helper() # should not stop here
+    """,
+    """
+    def helper_2():
+        pass # should not stop here
+    """,
+    """
+    def pdb_skipped_decorator2(function):
+        def wrapped_fn(*args, **kwargs):
+            __debuggerskip__ = True
+            helper_2()
+            __debuggerskip__ = False
+            result = function(*args, **kwargs)
+            __debuggerskip__ = True
+            helper_2()
+            return result
+        return wrapped_fn
+    """,
+    """
+    def pdb_skipped_decorator(function):
+        def wrapped_fn(*args, **kwargs):
+            __debuggerskip__ = True
+            helper_1()
+            __debuggerskip__ = False
+            result = function(*args, **kwargs)
+            __debuggerskip__ = True
+            helper_2()
+            return result
+        return wrapped_fn
+    """,
+    """
+    @pdb_skipped_decorator
+    @pdb_skipped_decorator2
+    def bar(x, y):
+        return x * y
+    """,
+    """import IPython.terminal.debugger as ipdb""",
+    """
+    def f():
+        ipdb.set_trace()
+        bar(3, 4)
+    """,
+    """
+    f()
+    """,
+)
+
+
+def _decorator_skip_setup():
+    import pexpect
+
+    env = os.environ.copy()
+    env["IPY_TEST_SIMPLE_PROMPT"] = "1"
+
+    child = pexpect.spawn(
+        sys.executable, ["-m", "IPython", "--colors=nocolor"], env=env
+    )
+    child.timeout = 15 * IPYTHON_TESTING_TIMEOUT_SCALE
+
+    child.expect("IPython")
+    child.expect("\n")
+
+    child.timeout = 5 * IPYTHON_TESTING_TIMEOUT_SCALE
+
+    dedented_blocks = [dedent(b).strip() for b in skip_decorators_blocks]
+    in_prompt_number = 1
+    for cblock in dedented_blocks:
+        child.expect_exact(f"In [{in_prompt_number}]:")
+        in_prompt_number += 1
+        for line in cblock.splitlines():
+            child.sendline(line)
+            child.expect_exact(line)
+        child.sendline("")
+    return child
+
+
+@skip_win32
+def test_decorator_skip():
+    """test that decorator frames can be skipped."""
+
+    child = _decorator_skip_setup()
+
+    child.expect_exact("3     bar(3, 4)")
+    child.expect("ipdb>")
+
+    child.expect("ipdb>")
+    child.sendline("step")
+    child.expect_exact("step")
+
+    child.expect_exact("1 @pdb_skipped_decorator")
+
+    child.sendline("s")
+    child.expect_exact("return x * y")
+
+    child.close()
+
+
+@pytest.mark.skipif(platform.python_implementation() == "PyPy", reason="issues on PyPy")
+@skip_win32
+def test_decorator_skip_disabled():
+    """test that decorator frame skipping can be disabled"""
+
+    child = _decorator_skip_setup()
+
+    child.expect_exact("3     bar(3, 4)")
+
+    for input_, expected in [
+        ("skip_predicates debuggerskip False", ""),
+        ("skip_predicates", "debuggerskip : False"),
+        ("step", "---> 2     def wrapped_fn"),
+        ("step", "----> 3         __debuggerskip__"),
+        ("step", "----> 4         helper_1()"),
+        ("step", "---> 1 def helper_1():"),
+        ("next", "----> 2     helpers_helper()"),
+        ("next", "--Return--"),
+        ("next", "----> 5         __debuggerskip__ = False"),
+    ]:
+        child.expect("ipdb>")
+        child.sendline(input_)
+        child.expect_exact(input_)
+        child.expect_exact(expected)
+
+    child.close()
+
+
+@pytest.mark.skipif(platform.python_implementation() == "PyPy", reason="issues on PyPy")
+@skip_win32
+def test_decorator_skip_with_breakpoint():
+    """test that decorator frame skipping can be disabled"""
+
+    import pexpect
+
+    env = os.environ.copy()
+    env["IPY_TEST_SIMPLE_PROMPT"] = "1"
+
+    child = pexpect.spawn(
+        sys.executable, ["-m", "IPython", "--colors=nocolor"], env=env
+    )
+    child.timeout = 15 * IPYTHON_TESTING_TIMEOUT_SCALE
+
+    child.expect("IPython")
+    child.expect("\n")
+
+    child.timeout = 5 * IPYTHON_TESTING_TIMEOUT_SCALE
+
+    ### we need a filename, so we need to exec the full block with a filename
+    with NamedTemporaryFile(suffix=".py", dir=".", delete=True) as tf:
+
+        name = tf.name[:-3].split("/")[-1]
+        tf.write("\n".join([dedent(x) for x in skip_decorators_blocks[:-1]]).encode())
+        tf.flush()
+        codeblock = f"from {name} import f"
+
+        dedented_blocks = [
+            codeblock,
+            "f()",
+        ]
+
+        in_prompt_number = 1
+        for cblock in dedented_blocks:
+            child.expect_exact(f"In [{in_prompt_number}]:")
+            in_prompt_number += 1
+            for line in cblock.splitlines():
+                child.sendline(line)
+                child.expect_exact(line)
+            child.sendline("")
+
+        # as the filename does not exists, we'll rely on the filename prompt
+        child.expect_exact("47     bar(3, 4)")
+
+        for input_, expected in [
+            (f"b {name}.py:3", ""),
+            ("step", "1---> 3     pass # should not stop here except"),
+            ("step", "---> 38 @pdb_skipped_decorator"),
+            ("continue", ""),
+        ]:
+            child.expect("ipdb>")
+            child.sendline(input_)
+            child.expect_exact(input_)
+            child.expect_exact(expected)
+
+    child.close()
+
+
 @skip_win32
 def test_where_erase_value():
     """Test that `where` does not access f_locals and erase values."""
