@@ -184,6 +184,7 @@ import re
 import shlex
 import sys
 import pygments
+from pathlib import Path
 
 from IPython.utils.text import marquee
 from IPython.utils import openpy
@@ -238,7 +239,7 @@ class Demo(object):
             terminal16m
 
           - style('default'): a string of pygments style name to be used.
-          """
+        """
         if hasattr(src, "read"):
              # It seems to be a file or a file-like object
             self.fname = "from a file-like object"
@@ -258,13 +259,17 @@ class Demo(object):
         self.auto_all = auto_all
         self.src = src
 
-        self.inside_ipython = "get_ipython" in globals()
+        try:
+            ip = get_ipython()  # this is in builtins whenever IPython is running
+            self.inside_ipython = True
+        except NameError:
+            self.inside_ipython = False
+
         if self.inside_ipython:
             # get a few things from ipython.  While it's a bit ugly design-wise,
             # it ensures that things like color scheme and the like are always in
             # sync with the ipython mode being used.  This class is only meant to
             # be used inside ipython anyways,  so it's OK.
-            ip = get_ipython()  # this is in builtins whenever IPython is running
             self.ip_ns       = ip.user_ns
             self.ip_colorize = ip.pycolorize
             self.ip_showtb   = ip.showtraceback
@@ -399,8 +404,8 @@ class Demo(object):
             index -= 1
 
         filename = self.shell.mktempfile(self.src_blocks[index])
-        self.shell.hooks.editor(filename,1)
-        with open(filename, 'r') as f:
+        self.shell.hooks.editor(filename, 1)
+        with open(Path(filename), "r", encoding="utf-8") as f:
             new_block = f.read()
         # update the source and colored block
         self.src_blocks[index] = new_block
@@ -527,7 +532,7 @@ class Demo(object):
                 elif token[0] == Token.Comment.Single:
                     toks.append((Token.Comment.Single, token[1][0]))
                     # parse comment content by rst lexer
-                    # remove the extrat newline added by rst lexer
+                    # remove the extra newline added by rst lexer
                     toks += list(pygments.lex(token[1][1:], self.rst_lexer))[:-1]
                 else:
                     toks.append(token)
